# Install required packages (rerun if R crashes)
# install.packages("timelineS")
# install.packages("lubridate")
# install.packages("ggplot2")
# install.packages("dplyr")
# install.packages("reshape2")
# install.packages("rPref")
# install.packages("mco")
# install.packages("MASS")


library(timelineS)
library(lubridate)
library(ggplot2)
library(dplyr)
library(reshape2)
library(rPref)
library(mco)
library(MASS)


rm(list = ls())

#Define plotting functions 
plot.timeline <- function(lifetime, events.name, start.Date, plot.Name) {
  dataP <- data.frame(Events = events.name, Event_Dates = ymd(start.Date) + years(lifetime))
  timelineS(dataP, main = plot.Name, labels = events.name, label.direction = "up", label.position = 3)
}

#Define distribution of events function
dist.Events <- function(lifetime, events, start.Date, option.Name) {
  events <- sort(events, decreasing = T)
  distribution.events <- sapply(events, seq, from = 0, to = lifetime)
  all.events <- melt(distribution.events)
  colnames(all.events) <- c("frequency", "event")
  all.events <- all.events[order(all.events$frequency),]
  unique.events <- all.events[!duplicated(all.events$frequency),]
  unique.events$event[which(unique.events$frequency == 0)] <- "DC"
 # plot.timeline(unique.events$frequency, unique.events$event, start.Date, option.Name) #added
  return(unique.events)
}


#P1 = Dam
#P2 = Steel Tank 
#P3 = Prim Sedimentation Tank 
#P4 = Pipes
#Combine lifetimes of products 1, 2
combine.lifeTimelines <- function(product.1, product.2, product.3, product.4, p1.dur, p2.dur, p3.dur, p4.dur) {
  base <- list(Names = c(), frequency = c(), duration = c())
  base$frequency <- sort(unique(c(product.1$frequency, product.2$frequency)), decreasing = FALSE)
  base$Names[1] <- "DC"
  base$duration[1] <- 0
  base$Names[length(base$frequency)] <- "END"
  base$duration[length(base$frequency)] <- 0
  for(index in 2:(length(base$frequency) - 1)) {
    phase.1 <- product.1$event[which(product.1$frequency == base$frequency[index])]
    phase.2 <- product.2$event[which(product.2$frequency == base$frequency[index])]
    phase.3 <- product.3$event[which(product.3$frequency == base$frequency[index])]
    phase.4 <- product.4$event[which(product.4$frequency == base$frequency[index])]
    #case 1: if both have maintenance, combine into 1 event
    if (sum(c(length(phase.1), length(phase.2), length(phase.3), length(phase.4)) != 0) >= 2) {
      base$Names[index] <- paste(na.omit(c(phase.1, phase.2, phase.3, phase.4)), collapse = ", ")
      base$duration[index] <- max(
        c(
          if (!is.null(phase.1)) p1.dur[which(names(p1.dur) == phase.1)] else NA,
          if (!is.null(phase.2)) p2.dur[which(names(p2.dur) == phase.2)] else NA,
          if (!is.null(phase.3)) p3.dur[which(names(p3.dur) == phase.3)] else NA,
          if (!is.null(phase.4)) p4.dur[which(names(p4.dur) == phase.4)] else NA
        ),
        na.rm = TRUE #this will make it remove NA values, maybe take out?
      )
      
      #case 2: only 1 product has an event 
    } else if(sum(c(length(phase.1), length(phase.2), length(phase.3), length(phase.4)) != 0) ==1) {
      base$Names[index] <- paste(na.omit(c(phase.1, phase.2, phase.3, phase.4)), collapse = ", ")
      base$duration[index] <- max(
        c(
          if (!is.null(phase.1)) p1.dur[which(names(p1.dur) == phase.1)] else NA,
          if (!is.null(phase.2)) p2.dur[which(names(p2.dur) == phase.2)] else NA,
          if (!is.null(phase.3)) p3.dur[which(names(p3.dur) == phase.3)] else NA,
          if (!is.null(phase.4)) p4.dur[which(names(p4.dur) == phase.4)] else NA
        ),
        na.rm = TRUE 
      )
      #case 3: no products have an event
    } else {
      base$duration[index] <- 0
    }
  }
  return(base)
}


# Define parameters
#par(mfrow = c(4, 1))
start.Date = "2025-01-01"
lifetime <- 100
events.Op1.Dam = c(M.Dam= 5, PR.Dam = 40, CR.Dam = 25, END = lifetime)
duration.ev.Dam <- c(M.Dam = 5, PR.Dam= 21, CR.Dam = 40) #edit
events.Op1.SteelTank <- c(M.ST= 6, J.ST = 25, RM.ST = 50, RCoat.ST = 20, END = lifetime)
duration.ev.SteelTank <- c(M.ST= 5, J.ST = 10, RM.ST = 20, RCoat.ST = 5)
events.Op1.PrimSedTank <- c(M.PST= 5, RCoat.PST = 20, MCoating.PST = 9, PRConc.PST = 10, TotRep.PST=50, END = lifetime)
duration.ev.PrimSedTank <- c(M.PST= 0.5, RCoat.PST = 5, MCoating.PST = 2, PRConc.PST = 7, TotRep.PST=60)
events.Op1.Pipes <<- c(Crack.Pipes= 10, J.Pipes = 10, RCoating.Pipes = 15, TotRep.Pipes=50, END = lifetime) 
duration.ev.Pipes <- c(Crack.Pipes= 7, J.Pipes = 5, RCoating.Pipes = 2, TotRep.Pipes=60) 
design.Options.Dam <- list(desing.Op1.Dam <- "Dam Prefabricated Crest and FRP Guide Walls")
design.Options.SteelTank <- list(desing.Op1.SteelTank <- "Steel Tank PRC Modular System")
design.Options.PrimSedTank <- list(desing.Op1.PrimSedTank <- "Primary Sedimentation Tank RC with Coating")
design.Options.Pipes <<- list(desing.Op1.Pipes <- "Pipes Reinforced Concrete")

# Maintenance schedules
maintenance.Dam <- dist.Events(lifetime, events.Op1.Dam, start.Date, design.Options.Dam)
maintenance.SteelTank <- dist.Events(lifetime, events.Op1.SteelTank, start.Date, design.Options.SteelTank)
maintenance.PrimSedTank <<- dist.Events(lifetime, events.Op1.PrimSedTank, start.Date, design.Options.PrimSedTank)
maintenance.Pipes <<- dist.Events(lifetime, events.Op1.Pipes, start.Date, design.Options.Pipes)


# Combine timelines
integrated.interv <- combine.lifeTimelines(maintenance.Dam, maintenance.SteelTank, maintenance.PrimSedTank, maintenance.Pipes, duration.ev.Dam, duration.ev.SteelTank, duration.ev.PrimSedTank, duration.ev.Pipes)

# Calculate total interruption duration
sum(integrated.interv$duration)

# Define design exploration function
design.explore <- function(events1, events2, events3, events4) {
  results <- c()
  for(i in 1:dim(events1)[1]) {
    ev1 <- unlist(events1[i, ])
    dist.1 <- dist.Events(lifetime, ev1, start.Date, desing.Op1.Dam)
    dur.ev1 <- ev1/2
    for (j in 1:dim(events1)[1]) {
      ev2 <- unlist(events2[j, ])
      dist.2 <- dist.Events(lifetime, ev2, start.Date, design.Options.SteelTank)
      dur.ev2 <- ev2/2
      for (k in 1:dim(events1)[1]) {
        ev3 <- unlist(events3[k, ])
        dist.3 <- dist.Events(lifetime, ev3, start.Date, design.Options.PrimSedTank)
        dur.ev3 <- ev3/2
        for (l in 1:dim(events1)[1]) {
          ev4 <<- unlist(events4[l, ])
          dist.4 <<- dist.Events(lifetime, ev4, start.Date, design.Options.Pipes)
          dur.ev4 <<- ev4/2
          combined.lifetime <- combine.lifeTimelines(dist.1, dist.2, dist.3, dist.4, dur.ev1, dur.ev2, dur.ev3, dur.ev4)
          min.dist.int <- min(abs(combined.lifetime$frequency[1:(length(combined.lifetime$frequency) - 1)] - combined.lifetime$frequency[2:length(combined.lifetime$frequency)]))
          results <- rbind(results, c(ev1, ev2, ev3, ev4, dur = sum(combined.lifetime$duration), dist.inter = min.dist.int))
        }
      }
    }
  }
  return(as.data.frame(results))
}

# Generate design space
n.grid <- 2 #increasing this means that the code takes a lot longer


events.grid.Dam <- expand.grid(M.Dam = sample(seq(2,7, by = 1), n.grid), 
                               PR.Dam = sample(seq(35,42,1), n.grid),
                               CR.Dam = sample(seq(18, 28, 1), n.grid))
events.grid.SteelTank <- expand.grid(M.ST = sample(seq(2, 9, by = 1), n.grid),
                                     J.ST = sample(seq(18,30,1), n.grid),
                                     RM.ST = sample(seq(40, 55, 1), n.grid),
                                     RCoat.ST = sample(seq(10,25,1), n.grid))
events.grid.PrimSedTank <- expand.grid(M.PST = sample(seq(1,7, by = 1), n.grid),
                                       RCoat.PST = sample(seq(14,26,1), n.grid),
                                       MCoating.PST = sample(seq(4, 11, 1), n.grid),
                                       PRConc.PST = sample(seq(6, 12, 1), n.grid),
                                       TotRep.PST = sample(seq(44, 53, 1), n.grid))
events.grid.Pipes <<- expand.grid(Crack.Pipes = sample(seq(6, 11, by = 1), n.grid),
                                  J.Pipes = sample(seq(5, 13, by = 1), n.grid), 
                                  RCoating.Pipes = sample(seq(9, 18, 1), n.grid),
                                  TotRep.Pipes = sample(seq(41, 53, 1), n.grid))

# Explore design space
response.space <- design.explore(events.grid.Dam, events.grid.SteelTank, events.grid.PrimSedTank, events.grid.Pipes)

# Define preferences and select best options
p <- low(dur) * high(dist.inter)
sky <- psel(response.space, p)
pareto2 <- psel(response.space, p, top = nrow(response.space))

# Visualize results
ggplot(response.space, aes(x = dur, y = dist.inter)) +
  geom_point(shape = 21) +
  geom_point(data = pareto2, size = 3, aes(color = factor(pareto2$.level)))

# Define function to show Pareto front with ggplot instead
show_front <- function(pref) {
  sky <- psel(response.space, pref)
  ggplot(response.space, aes(x = dur, y = dist.inter)) +
    geom_point(shape = 21) +
    geom_point(data = sky, aes(x = dur, y = dist.inter), color = "blue", size = 3) +
    ggtitle("Pareto Front with Preferences") +
    theme_minimal()
}

# Show Pareto front
show_front(p)

# Life Cycle Inventory and Analysis (change the directory to yours)
LCI.materials <- read.csv("/Users/sofijaradulovic/Downloads/TU Berlin/WLCSA/Group Project/LCIMaterials_GroupProject3_FinalCopy.csv")

#Initialize
epoxycoating.Q <- NA
foundation.volume <- NA
crest.volume <- NA
walls.volume <- NA
coating.Q <- NA
tank_volume <- NA
pipe1.volume <- NA
pipe2.volume <- NA

#User Inputs Dimensions
tank_depth <- 12 
tank_width <- 24 
tank_length <- 105 
sludge_length <- 7 
sludge_depth <- 5 

#All systems have two parts (tank and coating/dam crest and dam wall), so this is considered "subsystemPart1.Option" and "subsystemPart2.Option
LCA.System <- function(subsystemPart1.Option, subsystemPart2.Option, materials, dist.event) {
  materials.split <<- split(materials, materials$Scope)
  #s2 = frequency of events
  s <<- summary(as.factor(dist.event$event))
  
  s2 <<- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))
  if (!"J.Pipes" %in% colnames(s2)) {
    s2$J.Pipes <- 0
  }
  if (subsystemPart1.Option == "RCPST") {
    wall_thickness <- 0.25 #feet
    Vout <- (tank_length - sludge_length)*(tank_depth)*(tank_width) + (sludge_length)*(tank_depth + sludge_depth)*(tank_width)
    Vin1 <- sludge_length * tank_depth * (tank_width - 2*wall_thickness)
    Vin2 <- (sludge_depth - wall_thickness) * (sludge_length - 2*wall_thickness) * (tank_width - 2*wall_thickness)
    Vin <- (tank_length - sludge_length - wall_thickness)*(tank_depth - wall_thickness)*(tank_width - 2*wall_thickness) + Vin1 + Vin2
    tank_volume <- Vout - Vin
    #Assumes that for __ event, ___ amount of the original tank volume material is used:
    #Maintenance = 5%, Redo Coating = 100%, Partial Replacement = 10%, Total Replacement = 100%
    interventions.PSTtank <- 0.05 * s2$M.PST + s2$TotRep.PST
  } else if (subsystemPart1.Option == "PRCSteelTank"){
    st_length <- 12 # units: m  
    st_width <- 12 #units m  
    st_height <- 0.5 #units m 
    foundation.volume <- st_length * st_width * 0.6 * st_height   
    #Assuming: % of original materials for each _ event: M = 5% assumed less since not WW, but steel corrodes faster, PR = 5% (less than PST), TotRep = 100%
    interventions.foundation <- 0.05 * s2$M.ST + 0.10 * s2$RM.ST + 0.10 * s2$J.ST
  } else if (subsystemPart1.Option == "FRPDamWall") { 
    d.length <- 50 # units m
    d.width <- 20 #units m
    d.height <- 15 #units m
    d.thickness <- 1 #units m
    crest.volume <-  0.5 * d.length * d.width * d.thickness
    #Assumed: PR is more because you want the dam to not have interruptions as much due to letting all that water go
    #also important to maintain high level because it would be catastrophic if a dam broke
    #PR = 20%
    interventions.crest <- 0.20 * s2$PR.Dam + 0.05 * s2$M.Dam 
  } else if (subsystemPart1.Option == "Pipes1") { 
    p.length <- 1000 # units: m
    C.radius <- 0.360/2 #units m
    C.thickness <- 0.03 #units m
    pipe1.volume <-  pi * (C.radius^2 - (C.radius - C.thickness)^2) * p.length
    interventions.pipe1 <- 0.1 * s2$Crack.Pipes + (0.1 * s2$J.Pipes) + (0.1 * s2$RCoating.Pipes) + (0.01 * s2$TotRep.Pipes) 
  } 
  
  if (subsystemPart2.Option== "CoatingPST") {
    # Assumes every redo coating uses 100% original coating material
    interventions.PSTcoating <- s2$RCoat.PST + (0.15 * s2$MCoating.PST)
    coating_thickness <- 0.00328 #feet
    SA1 <- (tank_depth - wall_thickness)*(tank_width - 2*wall_thickness) + (tank_depth - wall_thickness)*(tank_length - sludge_length)*2 + (tank_length - sludge_length - wall_thickness)*(tank_width-2*wall_thickness)
    SA2 <- (tank_width - 2*wall_thickness)*(tank_depth + sludge_depth - wall_thickness) + (tank_length - 2*wall_thickness)*(tank_width-2*wall_thickness) + (sludge_length - 2*wall_thickness)*(sludge_depth - wall_thickness) + (sludge_length - 2*wall_thickness)*(tank_depth - sludge_depth - wall_thickness)*2
    tank_SA <- SA1 + SA2
    coating.Q <- coating_thickness * tank_SA
  } else if (subsystemPart2.Option == "CoatingSteelTank"){
    epthickness <- 0.002
    interventions.STcoating <- s2$RCoat.ST  #assumed 2 opposing effects cancel out: WW vs steel corrodes easier
    epoxycoating.Q <- 642.04*epthickness
  } else if (subsystemPart2.Option == "PRCDamCrest"){
    d.length <- 50 # units m
    d.width <- 20 #units m
    d.height <- 15 #units m
    d.thickness <- 1 #units m
    walls.volume <-  d.length  * d.height * 2 * 0.5 * 0.2
    #CR.Dam = 20% replaced each time
    interventions.walls <- 0.20 * s2$CR.Dam 
  } else if (subsystemPart2.Option == "Pipes2") { 
    pipe2.volume <-  1 #Material values are 0, this is kept 1 for code simplicity
    interventions.pipe2 <- 1 #Material values are 0, this is kept 1 for code simplicity
  }
  
  if (subsystemPart1.Option == "FRPDamWall" && subsystemPart2.Option == "PRCDamCrest") {
    Damcrest <- mutate(materials.split[[subsystemPart2.Option]],  system.Q = crest.volume, interventions = interventions.crest)
    Damwalls <- mutate(materials.split[[subsystemPart1.Option]], system.Q = walls.volume, interventions = interventions.walls)
    LCA.matrix <- rbind(Damcrest, Damwalls)
  }
  else if (subsystemPart1.Option == "PRCSteelTank" && subsystemPart2.Option == "CoatingSteelTank") {
    steelTepoxycoating <- mutate(materials.split[[subsystemPart2.Option]], system.Q = epoxycoating.Q, interventions =  interventions.STcoating) 
    steelTfoundation <- mutate(materials.split[[subsystemPart1.Option]],  system.Q = foundation.volume, interventions = interventions.foundation) 
    LCA.matrix <- rbind(steelTepoxycoating, steelTfoundation)
  }
  else if (subsystemPart1.Option == "RCPST" && subsystemPart2.Option == "CoatingPST"){
    PSTcoating <- mutate(materials.split[[subsystemPart2.Option]], system.Q = coating.Q, interventions = interventions.PSTcoating)
    PSTtank <- mutate(materials.split[[subsystemPart1.Option]], system.Q = (tank_volume), interventions = interventions.PSTtank)
    LCA.matrix <- rbind(PSTtank, PSTcoating)
  } 
  else if (subsystemPart1.Option == "Pipes1" && subsystemPart2.Option == "Pipes2") {
    Pipe1 <- mutate(materials.split[[subsystemPart1.Option]], system.Q = (pipe1.volume), interventions = interventions.pipe1)
    Pipe2 <- mutate(materials.split[[subsystemPart2.Option]], system.Q = (pipe2.volume), interventions = interventions.pipe2)
    print(Pipe1)
    LCA.matrix <<- rbind(Pipe1, Pipe2)
  }
  
  
  #LCA.matrix <- rbind(steelTepoxycoating, steelTfoundation, Damcrest, Damwalls, PSTcoating, PSTtank, Pipe1, Pipe2)
  
  # Check each intermediate calculation
  total_materials_q_check <- LCA.matrix$quantity * LCA.matrix$system.Q
  message("Your total_materials_q_check is:")
  print(total_materials_q_check)  # Check calculation for TotalMaterials.Q
  
  materials_lc_check <- total_materials_q_check * LCA.matrix$interventions
  message("Your materials_lc_check is:")
  print(materials_lc_check)  # Check calculation for materials.LC
  
  energy_lc_check <- materials_lc_check * LCA.matrix$energy
  message("Your energy_lc_check is:")
  print(energy_lc_check)  # Check calculation for Energy.LC
  
  co2_lc_check <- materials_lc_check * LCA.matrix$CO2
  message("Your co2_lc_check is:")
  print(co2_lc_check)  # Check calculation for CO2.LC
  
  nox_lc_check <- materials_lc_check * LCA.matrix$NOX
  message("Your nox_lc_check is:")
  print(nox_lc_check)  # Check calculation for NOx.LC
  
  so2_lc_check <- materials_lc_check * LCA.matrix$SO2
  message("Your SO2_lc_check is:")
  print(so2_lc_check)  # Check calculation for SO2.LC
  
  # Step 1: Calculate TotalMaterials.Q
  # print(any(sapply(quantity, is.null)))
  # print(any(sapply(system.Q, is.null)))
  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantity * system.Q)
  # print(LCA.matrix)
  # print(any(sapply(quantity, is.null)))
  # print(any(sapply(system.Q, is.null)))
  # Step 2: Calculate materials.LC
  LCA.matrix <- mutate(LCA.matrix, materials.LC = TotalMaterials.Q * interventions)
  # print(LCA.matrix)
  
  # Step 3: Calculate Energy.LC
  LCA.matrix <- mutate(LCA.matrix, Energy.LC = materials.LC * energy)
  # print(LCA.matrix)
  
  # Step 4: Calculate CO2.LC
  LCA.matrix <- mutate(LCA.matrix, CO2.LC = materials.LC * CO2)
  # print(LCA.matrix)
  
  # Step 5: Calculate NOx.LC
  LCA.matrix <- mutate(LCA.matrix, NOx.LC = materials.LC * NOX)
  # print(LCA.matrix)
  
  # Step 6: Calculate SO2.LC
  LCA.matrix <- mutate(LCA.matrix, SO2.LC = materials.LC * SO2)
  # print(LCA.matrix)
  # 
  # LCA.matrix <<- mutate(LCA.matrix, 
  #                       TotalMaterials.Q = quantity * system.Q, #total materials in kg
  #                       materials.LC = TotalMaterials.Q * interventions,
  #                       Energy.LC = materials.LC * energy,
  #                       CO2.LC = materials.LC * CO2, #emissions are already in kg
  #                       NOx.LC = materials.LC * NOX,
  #                       SO2.LC = materials.LC * SO2)
  
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOx = sum(LCA.matrix$NOx.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))
  
  return(LCA.results)
}

subsystemPart1.Option <- c("RCPST", "PRCSteelTank", "FRPDamWall", "Pipes1")
subsystemPart2.Option <- c("CoatingPST", "CoatingSteelTank", "PRCDamCrest", "Pipes2") #or none for pipes

# Perform LCA for Dam, Steel Tank, Primary Sedimentation Tank, and Pipes
Option.Dam <- LCA.System(subsystemPart1.Option[3], subsystemPart2.Option[3], LCI.materials, maintenance.Dam)
# Access LCA.matrix
# print(Option.Dam$LCA.matrix)
# Access LCA.results (for summary or other final results)
# print(Option.Dam$LCA.results)
Option.SteelTank <- LCA.System(subsystemPart1.Option[2], subsystemPart2.Option[2], LCI.materials, maintenance.SteelTank)
# Access LCA.matrix
# print(Option.SteelTank$LCA.matrix)

# # Access LCA.results (for summary or other final results)
# print(Option.SteelTank$LCA.results)
Option.PSTTank <- LCA.System(subsystemPart1.Option[1], subsystemPart2.Option[1], LCI.materials, maintenance.PrimSedTank)
# Access LCA.matrix
# print(Option.PSTTank$LCA.matrix)
# # Access LCA.results (for summary or other final results)
# print(Option.PSTTank$LCA.results)

Option.Pipes <- LCA.System(subsystemPart1.Option[4], subsystemPart2.Option[4], LCI.materials, maintenance.Pipes)
# print(Option.Pipes$LCA.matrix)
# print(Option.Pipes$LCA.results)

# # Check the structure of Option.Pipes
# str(Option.Pipes)
# 
# # Check if it contains the expected data
# print(Option.Pipes)
# Combine LCA results
integrated.Design <- as.data.frame(list(Energy = Option.Dam$Energy + Option.SteelTank$Energy + Option.PSTTank$Energy + Option.Pipes$Energy,
                                        CO2 = Option.Dam$CO2 + Option.SteelTank$CO2 + Option.PSTTank$CO2 + Option.Pipes$CO2,
                                        NOx = Option.Dam$NOx + Option.SteelTank$NOx + Option.PSTTank$NOx + Option.Pipes$NOx,
                                        SO2 = Option.Dam$SO2 + Option.SteelTank$SO2 + Option.PSTTank$SO2 + Option.Pipes$SO2))





# Define unit costs
Energy.costs <- 0.128
CO2.unitcost <- 26
NOx.unitCost <- 42
SO2.unitCosts <- 85

# Calculate total costs
integrated.Design <- mutate(integrated.Design, 
                            Costs = (Energy * Energy.costs + CO2*CO2.unitcost + NOx * NOx.unitCost + SO2 * SO2.unitCosts)/10^9)


#________________MOO______
# MultiObjective Optimization
fitness <- function(x) {
  z <- numeric(7)
  x <<- round(x, 0)
  y <<- expand.grid(M.Dam = x[1], PR.Dam = x[2], CR.Dam = x[3], 
                    M.ST = x[4], J.ST = x[5], RM.ST= x[6], RCoat.ST = x[7],
                    M.PST = x[8], RCoat.PST = x[9], MCoating.PST = x[10], PRConc.PST = x[11], TotRep.PST= x[12],
                    Crack.Pipes= x[13], J.Pipes = x[14], RCoating.Pipes = x[15], TotRep.Pipes= x[16])
  
  #Apply 1:3 because these are the maintenance events for the first product
  dur.ev1 <- unlist(y[1:3] / 2)
  dur.ev2 <- unlist(y[4:7] / 2)
  dur.ev3 <- unlist(y[8:12] / 2)
  dur.ev4 <- unlist(y[13:16] / 2)
  dist.1 <- apply(y[1:3], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date, design.Options.Dam)
  dist.2 <- apply(y[4:7], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date, design.Options.SteelTank)
  dist.3 <<- apply(y[8:12], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date, design.Options.PrimSedTank)
  dist.4 <- apply(y[13:16], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date, design.Options.Pipes)
  results <- combine.lifeTimelines(dist.1[[1]], dist.2[[1]], dist.3[[1]], dist.4[[1]], dur.ev1, dur.ev2, dur.ev3, dur.ev4)
  z[1] <- sum(results[["duration"]])
  z[2] <- -min(abs(results$frequency[1:(length(results$frequency) - 1)] - results$frequency[2:length(results$frequency)]))
  
  # b.width <- x[7]
  # b.length <- 25
  # bd.depth <- 0.35
  # asphalt.tk <- 0.12
  # r.depth <- 0.5
  # r.lenght <- 200
  # materials <- LCI.materials
  # girder.Options <- c("PRC", "none")
  # deck.options <- c("RC", "SteelTank")
  
  subsystemPart1.Option <- c("RCPST", "PRCSteelTank", "FRPDamWall", "Pipes")
  subsystemPart2.Option <- c("CoatingPST", "CoatingSteelTank", "PRCDamCrest", "Pipes") #or none for pipes
  
  # # Perform LCA for Dam, Steel Tank, Primary Sedimentation Tank, and Pipes
  # Option.Dam <- LCA.System(subsystemPart1.Option[3], subsystemPart2.Option[3], LCI.materials, maintenance.Dam)
  # Option.SteelTank <- LCA.System(subsystemPart1.Option[2], subsystemPart2.Option[2], LCI.materials, maintenance.SteelTank)
  # Option.PSTTank <- LCA.System(subsystemPart1.Option[1], subsystemPart2.Option[1], LCI.materials, maintenance.PrimSedTank)
  # Option.Pipes <- LCA.System(subsystemPart1.Option[4], subsystemPart2.Option[4], LCI.materials, maintenance.Pipes)
  # 
  product.1.Dam <- LCA.System(subsystemPart1.Option[3], subsystemPart2.Option[3], LCI.materials, dist.1[[1]])
  product.2.SteelTank <- LCA.System(subsystemPart1.Option[2], subsystemPart2.Option[2], LCI.materials, dist.2[[1]])
  product.3.PST <- LCA.System(subsystemPart1.Option[1], subsystemPart2.Option[1], LCI.materials, dist.3[[1]])
  product.4.Pipes <- LCA.System(subsystemPart1.Option[4], subsystemPart2.Option[4], LCI.materials, dist.4[[1]])
  
  integrated.system <<- as.data.frame(list(Energy = product.1.Dam$Energy + product.2.SteelTank$Energy + product.3.PST$Energy + product.4.Pipes$Energy,
                                           CO2 = product.1.Dam$CO2 + product.2.SteelTank$CO2 + product.3.PST$CO2 + product.4.Pipes$CO2,
                                           NOx = product.1.Dam$NOx + product.2.SteelTank$NOx + product.3.PST$NOx + product.4.Pipes$NOx,
                                           SO2 = product.1.Dam$SO2 + product.2.SteelTank$SO2 + product.3.PST$SO2 + product.4.Pipes$SO2))
  
  integrated.system <- mutate(integrated.system, 
                              Costs = (Energy * Energy.costs + CO2*CO2.unitcost + NOx * NOx.unitCost + SO2 * SO2.unitCosts)/10^9)
  
  z[3] <- integrated.system$Energy
  z[4] <- integrated.system$CO2
  z[5] <- integrated.system$NOx
  z[6] <- integrated.system$SO2
  z[7] <- integrated.system$Costs
  
  return(z)
}
fitness0 <- function(x) {
  z <- numeric(2)
  x <<- round(x, 0)
  y <<- expand.grid(M.Dam = x[1], PR.Dam = x[2], CR.Dam = x[3], 
                    M.ST = x[4], J.ST = x[5], RM.ST= x[6], RCoat.ST = x[7],
                    M.PST = x[8], RCoat.PST = x[9], MCoating.PST = x[10], PRConc.PST = x[11], TotRep.PST= x[12],
                    Crack.Pipes= x[13], J.Pipes = x[14], RCoating.Pipes = x[15], TotRep.Pipes= x[16])
  
  #Apply 1:3 because these are the maintenance events for the first product
  dur.ev1 <- unlist(y[1:3] / 2)
  dur.ev2 <- unlist(y[4:7] / 2)
  dur.ev3 <- unlist(y[8:12] / 2)
  dur.ev4 <- unlist(y[13:16] / 2)
  dist.1 <- apply(y[1:3], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date, design.Options.Dam)
  dist.2 <- apply(y[4:7], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date, design.Options.SteelTank)
  dist.3 <<- apply(y[8:12], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date, design.Options.PrimSedTank)
  dist.4 <- apply(y[13:16], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date, design.Options.Pipes)
  results <- combine.lifeTimelines(dist.1[[1]], dist.2[[1]], dist.3[[1]], dist.4[[1]], dur.ev1, dur.ev2, dur.ev3, dur.ev4)
  z[1] <- sum(results[["duration"]])
  # z[2] <- -min(abs(results$frequency[1:(length(results$frequency) - 1)] - results$frequency[2:length(results$frequency)]))
  
  subsystemPart1.Option <- c("RCPST", "PRCSteelTank", "FRPDamWall", "Pipes")
  subsystemPart2.Option <- c("CoatingPST", "CoatingSteelTank", "PRCDamCrest", "Pipes") #or none for pipes
  
 
  product.1.Dam <- LCA.System(subsystemPart1.Option[3], subsystemPart2.Option[3], LCI.materials, dist.1[[1]])
  product.2.SteelTank <- LCA.System(subsystemPart1.Option[2], subsystemPart2.Option[2], LCI.materials, dist.2[[1]])
  product.3.PST <- LCA.System(subsystemPart1.Option[1], subsystemPart2.Option[1], LCI.materials, dist.3[[1]])
  product.4.Pipes <- LCA.System(subsystemPart1.Option[4], subsystemPart2.Option[4], LCI.materials, dist.4[[1]])
  
  integrated.system <<- as.data.frame(list(Energy = product.1.Dam$Energy + product.2.SteelTank$Energy + product.3.PST$Energy + product.4.Pipes$Energy,
                                           CO2 = product.1.Dam$CO2 + product.2.SteelTank$CO2 + product.3.PST$CO2 + product.4.Pipes$CO2,
                                           NOx = product.1.Dam$NOx + product.2.SteelTank$NOx + product.3.PST$NOx + product.4.Pipes$NOx,
                                           SO2 = product.1.Dam$SO2 + product.2.SteelTank$SO2 + product.3.PST$SO2 + product.4.Pipes$SO2))
  
  integrated.system <- mutate(integrated.system, 
                              Costs = (Energy * Energy.costs + CO2*CO2.unitcost + NOx * NOx.unitCost + SO2 * SO2.unitCosts)/10^9)
  
  # z[3] <- integrated.system$Energy
  # z[4] <- integrated.system$CO2
  # z[5] <- integrated.system$NOx
  # z[6] <- integrated.system$SO2
  z[2] <- integrated.system$Costs
  
  return(z)
}
#idim = input dimension = maintenance 
#odim = output dimension 



r2_0 <- nsga2(fitness0, idim = 16, odim = 2,
              generations = 20, popsize = 100,
              lower.bounds = c(2, 35, 18, 2, 18, 40, 10, 1, 14, 4, 6, 44, 6, 5, 9, 41),
              upper.bounds = c(7, 42, 28, 9, 30, 55, 25, 7, 26, 11, 12, 53, 11, 13, 18, 53))

# Run NSGA-II optimization
r2 <- nsga2(fitness, idim = 16, odim = 7,
            generations = 20, popsize = 100,
            lower.bounds = c(2, 35, 18, 2, 18, 40, 10, 1, 14, 4, 6, 44, 6, 5, 9, 41),
            upper.bounds = c(7, 42, 28, 9, 30, 55, 25, 7, 26, 11, 12, 53, 11, 13, 18, 53))
# Transform results to dataframes
r2Results <- as.data.frame(r2$value)
outNames <- c("duration", "interv.dist", "energy", "co2", "nox", "so2", "cost")
colnames(r2Results) <- outNames
r2Results0 <- as.data.frame(r2_0$value)
outNames0 <- c("duration","cost")
colnames(r2Results0) <- outNames0

# Extract Pareto fronts
pareto3 <- as.data.frame(paretoFront(r2))
colnames(pareto3) <- outNames
pareto3_0 <- as.data.frame(paretoFront(r2_0))
colnames(pareto3_0) <- outNames0

# Plot Pareto front (duration vs cost)
ggplot(r2Results, aes(x = duration, y = cost)) + 
  geom_point(shape = 21) +
  geom_point(data = pareto3, size = 3, color="red") + 
  geom_line(data = pareto3, color="blue") +
  labs(title = "Pareto Front: Duration vs Cost",
       x = "Total Duration of Interventions",
       y = "Total Cost")

# Plot Pareto front (duration vs cost)
ggplot(r2Results0, aes(x = duration, y = cost)) + 
  geom_point(shape = 21) +
  geom_point(data = pareto3_0, size = 3, color="red") + 
  geom_line(data = pareto3_0, color="blue") +
  labs(title = "Pareto Front: Duration vs Cost",
       x = "Total Duration of Interventions",
       y = "Total Cost")

# Prepare data for parallel coordinate plot
input.params <- round(r2$par, 0)
all.results <- cbind(input.params, r2Results, r2$pareto.optimal)
colnames(all.results) <- c("pareto","M.Dam", "PR.Dam", "CR.Dam", 
                           "M.ST", "J.ST", "RM.ST", "RCoat.ST",
                           "M.PST", "RCoat.PST", "MCoating.PST", "TotRep.PST",
                           "Crack.Pipes", "J.Pipes", "RCoating.Pipes", "TotRep.Pipes",
                           "Duration", "Interv.Dist",
                           "Energy", "CO2", "NOx", "SO2", "CostEmissions", "TRUEFALSE")


#Will probably need to edit the lty lwd etc. already edited [, 1:21]
# Create parallel coordinate plot
# par(mfrow = c(1,1))
# parcoord(all.results[, 1:21], var.label = TRUE,
#          col = ifelse(all.results$pareto == TRUE, "indianred", "skyblue2"), 
#          lty = ifelse(all.results$pareto == TRUE, 1, 3),
#          lwd = ifelse(all.results$pareto == TRUE, 3, 1))
# title("Parallel Coordinate Plot of Design Parameters and Objectives")

par(mfrow = c(1,1))
parcoord(all.results[, 2:22], var.label = TRUE,
         col = ifelse(all.results$TRUEFALSE == TRUE, "indianred", "skyblue2"), 
         lty = ifelse(all.results$TRUEFALSE == TRUE, 1, 3),
         lwd = ifelse(all.results$TRUEFALSE == TRUE, 3, 1))
title("Parallel Coordinate Plot of Design Parameters and Objectives")

# Print summary of Pareto-optimal solutions
cat("\nSummary of Pareto-optimal solutions:\n")
print(summary(pareto3))

# Print best solution for each objective
cat("\nBest solutions for each objective:\n")
for(obj in outNames) {
  if(obj == "interv.dist") {
    best <- pareto3[which.max(pareto3[[obj]]),]
  } else {
    best <- pareto3[which.min(pareto3[[obj]]),]
  }
  cat(paste0("Best ", obj, ":\n"))
  print(best)
  cat("\n")
}

# Save results
write.csv(pareto3, "pareto_optimal_solutions.csv", row.names = FALSE)